//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of CPCtelera: An Amstrad CPC Game Engine
//  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include "main.h"
#include "menu.h"
#include "game.h"
#include "PlayerAKG.h"
#include "fonts2B.h"
#include "graphics.h"
#include "render.h"
#include "consola.h"
#include "intro1.h"
#include "intro2.h"
extern u8* SFX_SOUNDEFFECTS;
extern u8* MENU_START;

const u8 version = 0x04;

u8 paleta[16];
const u8 paleta_original[16] =   { HW_BLACK, HW_WHITE, HW_BRIGHT_WHITE ,HW_PINK, 
                          HW_YELLOW, HW_RED, HW_MAGENTA, HW_MAUVE, 
                          HW_PASTEL_BLUE, HW_BRIGHT_RED, HW_BRIGHT_YELLOW, HW_PURPLE,
                          HW_BRIGHT_CYAN, HW_BRIGHT_GREEN, HW_GREEN, HW_ORANGE};
const u8 paleta_oscura[16] =   { HW_BLACK, HW_BLACK, HW_WHITE ,HW_RED, 
                          HW_BLACK, HW_BLACK, HW_RED, HW_RED, 
                          HW_BLUE, HW_RED, HW_YELLOW, HW_RED,
                          HW_BLUE, HW_GREEN, HW_GREEN, HW_RED};                          
//const u8 paleta[16] =   { 0x54, 0x55, 0x4C ,0x52, 0x56, 0x44, 0x45, 0x5C, 0x5D, 0x5F, 0x40, 0x4A, 0x47, 0x4E, 0x53, 0x4B};

u8* mem_start;
u8 mem_page;
u8 swap_memvideo;
u8 musica;

TEntidad entidades[MAX_ENTIDADES];
u8 ciclo;
u8 mapa[20*23]; //El escenario es de 23 filas con 20 tiles cada una; o sea, 460 Bytes
u8 finFase;

u8 habitacion;
u8 laberinto_hechos[(LABERINTO_COLUMNAS*LABERINTO_FILAS)/8];
u8 laberinto[(LABERINTO_COLUMNAS*LABERINTO_FILAS)/2];
u8 hab_jefes[2];
u8 tarjetas[3];

const u8 laberinto1[(LABERINTO_COLUMNAS*LABERINTO_FILAS)/2] =
{
 0b01010011, 0b01100101, 0b01100100, 0b01010110
,0b10000101, 0b11111010, 0b10001101, 0b10101100
,0b00011010, 0b10010011, 0b00111010, 0b01011010
,0b01010011, 0b00110110, 0b01010110, 0b10010110
,0b10010110, 0b00011111, 0b11101001, 0b01101100
,0b01011010, 0b00011010, 0b10000001, 0b10111010
,0b10010111, 0b00100101, 0b00110011, 0b01100100
,0b00011011, 0b01111011, 0b00100101, 0b11101100
,0b00010011, 0b10110010, 0b01011010, 0b10011010
};
/*
ESTRUCTURA DEL MAPEADO 1 (COMPROBADO)

*-*-* S-* * *-*   0-7
|   | | | | | |
R *-*-* * *-* *   8-15
  | |     |   |
*-* *-*-*-* *-*   16-23
            |
*-*-*-* *-* *-*   24-31
|     | | |   |
*-* *-*-* *-* *   32-39
  |   | |   | |
*-* *-* * B-*-*   40-47
|           
*-*-* *-*-*-* *   48-55
  |   |     | |
*-*-*-*-* *-* *   56-63
    |     | | |
G-*-*-* E-* *-*   64-71


0000 = Cerrado todas las puertas
1111 = Abiertas: Arriba, Abajo, Izquierda, Derecha (en ese orden)
*/

const u8 laberinto2[(LABERINTO_COLUMNAS*LABERINTO_FILAS)/2] =
{
 0b01000101, 0b01100001, 0b01110110, 0b00010110
,0b10011110, 0b11000101, 0b10101101, 0b00111110
,0b01011010, 0b10011011, 0b00111010, 0b00011010
,0b11000001, 0b01110110, 0b01010110, 0b00010110
,0b11010111, 0b10111111, 0b10101001, 0b01111110
,0b10011010, 0b00011010, 0b01010011, 0b10111010
,0b00010110, 0b00010110, 0b10010011, 0b01100100
,0b01001101, 0b01111011, 0b00110110, 0b11001100
,0b10011011, 0b10110011, 0b01101001, 0b10111010
};
/*
ESTRUCTURA DEL MAPEADO 2 (COMPROBADO)

* *-* S-*-* R-*   0-7
| | |   | |   |
*-* * *-* *-*-*   8-15
  | | |   |   |
*-* *-*-*-* *-*   16-23
|           
* G-*-* *-* *-*   24-31
|   | | | |   |
*-*-*-*-* *-*-*   32-39
| |   |     | |
*-* *-* *-*-*-*   40-47
        |    
B-* *-* *-*-* *   48-55
  |   |     | |
* *-*-*-*-* * *   56-63
| | |     | | |
*-*-*-*-E *-*-*   64-71


0000 = Cerrado todas las puertas
1111 = Abiertas: Arriba, Abajo, Izquierda, Derecha (en ese orden)
*/

const u8 laberinto3[(LABERINTO_COLUMNAS*LABERINTO_FILAS)/2] =
{
 0b01010011, 0b00110011, 0b01110111, 0b00110110
,0b11010110, 0b00010011, 0b10101100, 0b00011110
,0b10011010, 0b00010011, 0b00111010, 0b01011010
,0b01010011, 0b00100101, 0b01100101, 0b10110110
,0b11010111, 0b00111110, 0b10011011, 0b01101100
,0b10011010, 0b00011011, 0b00100001, 0b10111110
,0b00010110, 0b00010110, 0b00010111, 0b01101100
,0b01011111, 0b01111011, 0b01111110, 0b11001100
,0b10001001, 0b10110010, 0b11001000, 0b10011010
};

/*
ESTRUCTURA DEL MAPEADO 3 (COMPROBADO)

*-*-*-S-*-*-*-*   0-7
|       | |   |
*-* R-*-* * *-*   8-15
| |       |   |
*-* *-*-*-* *-*   16-23
            |
*-*-G *-* *-*-*   24-31
|     | | |   |
*-*-*-* *-*-* *   32-39
| |   |     | |
*-* *-*-* *-*-*   40-47
              |
B-* *-* *-*-* *   48-55
  |   |   | | |
*-*-*-*-*-* * *   56-63
| | |   | | | |
* *-*-* E * *-*   64-71


0000 = Cerrado todas las puertas
1111 = Abiertas: Arriba, Abajo, Izquierda, Derecha (en ese orden)
*/

const u8 hab_jefes1[2] = {22,40};
const u8 hab_jefes2[2] = {16,44};
const u8 hab_jefes3[2] = {22,47};

const u8 tarjetas1[3] = {8,45,64};
const u8 tarjetas2[3] = {6,25,48};
const u8 tarjetas3[3] = {10,26,48};

//Diseño de Enemigos
void initEnemigos() __naked{ 
//Inicializar los datos de los enemigos directamente en memoria, sin usar asignaciones
//u8 id;   u8 x;   u8 y; i8 data1
//
//id 2=Droide base 
//x = tile columna * 4
//y = tile fila * 8
   __asm
   _enemigos::
      //00
      .db TIPO_EJES, 32, 80, 2
      .db TIPO_EJES, 44, 120, 2
      .db 255,0,0,0  
      //01
      .db TIPO_CANNON, 72, 24, 2
      .db TIPO_CANNON, 4, 24, 2
      .db 255,0,0,0  
      //02
      .db TIPO_PERS_ESTATICO, 14, 44, 4
      .db TIPO_EJES, 60, 144, 3
      .db 255,0,0,0  
      //03
      .db TIPO_EJES, 38, 116, 2
      .db TIPO_EJES, 38, 116, 3
      .db 255,0,0,0  
      //04
      .db TIPO_PERSEGUIDOR, 38, 40, 5
      .db TIPO_ALEATORIO, 38, 24, 50
      .db 255,0,0,0 
      //05
      .db TIPO_CONCURSANTE_2, 38, 100, 3
      .db 255,0,0,0 
      //06
      .db TIPO_PERS_ESTATICO, 4, 24, 6
      .db TIPO_PERS_ESTATICO, 72, 176, 5
      .db 255,0,0,0 
      //07
      .db TIPO_EJES, 6, 24, 3
      .db 255,0,0,0 
      //08
      .db TIPO_ALEATORIO, 42, 100, 30
      .db TIPO_EJES, 36,160,3
      .db 255,0,0,0 
      //09
      .db TIPO_ALEATORIO, 38,100,40
      .db TIPO_DISPARO, 38, 100, 3
      .db 255,0,0,0 
      //10
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db TIPO_ALEATORIO, 56, 24, 20
      .db 255,0,0,0 
      //11
      .db TIPO_EJES, 38, 100, 1
      .db TIPO_EJES, 38, 92, 2
      .db TIPO_EJES, 38, 124, 2
      .db 255,0,0,0 
      //12
      .db TIPO_PERS_ESTATICO, 38, 100, 3
      .db TIPO_CANNON, 72, 24, 3
      .db 255,0,0,0 
      //13
      .db TIPO_EJES, 38, 100, 3   
      .db TIPO_ALEATORIO, 38, 108, 10
      .db 255,0,0,0 
      //14
      .db TIPO_CANNON, 72, 24, 3
      .db TIPO_EJES, 38, 80, 3
      .db TIPO_EJES, 38, 120, 3
      .db 255,0,0,0 
      //15
      .db TIPO_DISPARO, 28, 100, 3
      .db TIPO_DISPARO, 48, 100, 3
      .db 255,0,0,0 
      //16
      .db TIPO_CANNON, 72, 72, 1
      .db TIPO_CANNON, 72,124, 1
      .db 255,0,0,0 
      //17
      .db TIPO_EJES, 38, 100, 1
      .db TIPO_CONCURSANTE_1, 38,140, 1
      .db 255,0,0,0 
      //18
      .db TIPO_EJES, 38, 160, 3
      .db TIPO_PERSEGUIDOR, 38, 140, 5
      .db 255,0,0,0 
      //19
      .db TIPO_PERSEGUIDOR, 38, 100, 3
      .db TIPO_EJES, 38, 100, 2
      .db TIPO_EJES, 38, 100, 1
      .db 255,0,0,0 
      //20
      .db TIPO_PERS_PATRULLA, 38, 100, 4
      .db TIPO_ALEATORIO, 4, 24, 5
      .db 255,0,0,0 
      //21
      .db TIPO_CANNON, 72, 24, 3
      .db TIPO_CANNON, 72, 24, 2
      .db 255,0,0,0 
      //22
      .db TIPO_EJES, 38, 80, 1
      .db TIPO_EJES, 38, 120, 1
      .db TIPO_EJES, 38, 100, 2
      .db 255,0,0,0 
      //23
      .db TIPO_CANNON, 72, 24, 2
      .db TIPO_CANNON, 4, 24, 2
      .db TIPO_ALEATORIO, 38, 60, 20
      .db 255,0,0,0 


      //24
      .db TIPO_EJES, 32, 80, 2
      .db TIPO_EJES, 44, 120, 2
      .db TIPO_DISPARO, 38, 80, 3
      .db 255,0,0,0  
      //25
      .db TIPO_CANNON, 4, 24, 2
      .db TIPO_PERSEGUIDOR, 38, 100, 3
      .db 255,0,0,0  
      //26
      .db TIPO_DISPARO, 50, 80, 3
      .db TIPO_EJES, 38, 100, 2
      .db TIPO_PINCHOS, 38, 100, 0
      .db 255,0,0,0  
      //27
      .db TIPO_CONCURSANTE_2, 38, 100, 3
      .db TIPO_PERSEGUIDOR, 38, 40, 5
      .db 255,0,0,0  
      //28
      .db TIPO_CANNON, 72, 128, 3
      .db TIPO_ALEATORIO, 38, 24, 50
      .db 255,0,0,0 
      //29
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db 255,0,0,0 
      //30
      .db TIPO_PINCHOS, 38, 100, 0
      .db TIPO_PINCHOS, 38, 100, 0
      .db TIPO_ALEATORIO, 38, 100, 8
      .db 255,0,0,0 
      //31
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db 255,0,0,0 
      //32
      .db TIPO_ALEATORIO, 42, 100, 10
      .db TIPO_ALEATORIO, 6,160,5
      .db 255,0,0,0 
      //33
      .db TIPO_EJES, 38, 100, 1
      .db TIPO_PERS_ESTATICO, 70, 40, 6
      .db TIPO_PERS_ESTATICO, 8, 160, 6
      .db 255,0,0,0 
      //34
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db TIPO_PINCHOS, 38, 100, 0
      .db TIPO_PINCHOS, 38, 100, 0
      .db 255,0,0,0 
      //35
      .db TIPO_EJES, 38, 100, 1
      .db TIPO_EJES, 40, 100, 2
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db 255,0,0,0 
      //36
      .db TIPO_PERSEGUIDOR, 28, 100, 3
      .db TIPO_CANNON, 72, 128, 3
      .db TIPO_CANNON, 72, 24, 2
      .db 255,0,0,0 
      //37
      .db TIPO_EJES, 38, 100, 3   
      .db TIPO_EJES, 38, 88, 1
      .db TIPO_PINCHOS, 38, 100, 0
      .db 255,0,0,0 
      //38
      .db TIPO_CANNON, 72, 128, 3
      .db TIPO_EJES, 38, 40, 3
      .db TIPO_EJES, 38, 80, 3
      .db TIPO_EJES, 38, 120, 3
      .db 255,0,0,0 
      //39
      .db TIPO_POSTES, 24, 72, 80
      .db TIPO_POSTES, 24,152, 1
      .db TIPO_DISPARO, 28, 100, 3
      .db TIPO_DISPARO, 48, 100, 3
      .db 255,0,0,0 
      //40
      .db TIPO_CANNON, 72, 76, 3
      .db TIPO_CONCURSANTE_2, 38,140, 4
      .db 255,0,0,0 
      //41
      .db TIPO_EJES, 38, 100, 1
      .db TIPO_DISPARO, 38, 80, 3
      .db TIPO_PERSEGUIDOR, 38, 120, 4
      .db 255,0,0,0 
      //42
      .db TIPO_POSTES, 18, 32, 80
      .db TIPO_POSTES, 18,112, 1
      .db TIPO_EJES, 38, 160, 3
      .db 255,0,0,0 
      //43
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db TIPO_EJES, 38, 100, 2
      .db TIPO_EJES, 38, 100, 1
      .db 255,0,0,0 
      //44
      .db TIPO_DISPARO, 38, 100, 3
      .db TIPO_EJES, 4, 24, 3
      .db 255,0,0,0 
      //45
      .db TIPO_CANNON, 72, 128, 3
      .db TIPO_CANNON, 72, 24, 2
      .db TIPO_CONCURSANTE_2, 38,140, 3
      .db 255,0,0,0 
      //46
      .db TIPO_PINCHOS, 38, 100, 0
      .db TIPO_DISPARO, 38, 100, 3
      .db 255,0,0,0 
      //47
      .db TIPO_CANNON, 72, 24, 2
      .db TIPO_CANNON, 4, 24, 2
      .db TIPO_CANNON, 72, 128, 3
      .db TIPO_EJES, 38, 80, 3
      .db 255,0,0,0 



      //48
      .db TIPO_POSTES, 20, 24, 80
      .db TIPO_POSTES, 20,104, 1
      .db TIPO_POSTES, 56,96, 90
      .db TIPO_POSTES, 56,176, 1
      .db TIPO_ALEATORIO, 4, 24, 30
      .db 255,0,0,0  
      //49
      .db TIPO_DISPARO, 38, 100, 3
      .db TIPO_EJES, 8, 24, 3
      .db 255,0,0,0  
      //50
      .db TIPO_PINCHOS, 38, 100, 0
      .db TIPO_PINCHOS, 38, 100, 0
      .db TIPO_EJES, 20, 56, 3
      .db TIPO_EJES, 60, 144, 3
      .db 255,0,0,0  
      //51
      .db TIPO_CONCURSANTE_2, 38, 100, 3
      .db TIPO_CONCURSANTE_1, 38, 60, 1
      .db 255,0,0,0  
      //52
      .db TIPO_PERS_PATRULLA, 4, 32, 5
      .db TIPO_DISPARO, 38, 24, 50
      .db 255,0,0,0 
      //53
      .db TIPO_EJES, 38, 100, 1
      .db TIPO_EJES, 38, 100, 2
      .db TIPO_PINCHOS, 38, 100, 0
      .db 255,0,0,0 
      //54
      .db TIPO_PERS_PATRULLA, 38, 100, 3
      .db TIPO_POSTES, 38, 64, 90
      .db TIPO_POSTES, 38,144, 1
      .db TIPO_PINCHOS, 38, 100, 0
      .db 255,0,0,0 
      //55
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db TIPO_PINCHOS, 38, 100, 0
      .db TIPO_PINCHOS, 38, 100, 0
      .db 255,0,0,0 
      //56
      .db TIPO_ALEATORIO, 6,160,5
      .db TIPO_ALEATORIO, 42, 100, 10
      .db TIPO_ALEATORIO, 68, 40, 3
      .db 255,0,0,0 
      //57
      .db TIPO_PERS_ESTATICO, 70, 40, 4
      .db TIPO_PERS_ESTATICO, 8, 160, 4
      .db TIPO_POSTES, 38, 64, 90
      .db TIPO_POSTES, 38,144, 1
      .db 255,0,0,0 
      //58
      .db TIPO_ALEATORIO, 26, 100, 1
      .db TIPO_ALEATORIO, 50, 100, 2
      .db TIPO_PINCHOS, 38, 100, 0
      .db 255,0,0,0 
      //59
      .db TIPO_DISPARO, 38, 100, 1
      .db TIPO_DISPARO, 40, 100, 2
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db 255,0,0,0 
      //60
      .db TIPO_PERSEGUIDOR, 38, 100, 3
      .db TIPO_CANNON, 72, 128, 3
      .db TIPO_CANNON, 72, 24, 2
      .db 255,0,0,0 
      //61
      .db TIPO_ALEATORIO, 38, 100, 10   
      .db TIPO_EJES, 38, 84, 1
      .db TIPO_PINCHOS, 38, 100, 0
      .db 255,0,0,0 
      //62
      .db TIPO_CANNON, 72, 128, 3
      .db TIPO_EJES, 38, 40, 3
      .db TIPO_EJES, 38, 80, 3
      .db TIPO_EJES, 38, 120, 3
      .db TIPO_EJES, 38, 160, 3
      .db 255,0,0,0 
      //63
      .db TIPO_POSTES, 24, 72, 80
      .db TIPO_POSTES, 24,152, 1
      .db TIPO_POSTES, 52, 48, 80
      .db TIPO_POSTES, 52,128, 1
      .db TIPO_EJES, 28, 100, 3
      .db TIPO_EJES, 48, 100, 3
      .db 255,0,0,0 
      //64
      .db TIPO_CANNON, 72, 76, 3
      .db TIPO_CONCURSANTE_1, 38,140, 1
      .db 255,0,0,0 
      //65
      .db TIPO_EJES, 38, 100, 1
      .db TIPO_EJES, 38, 100, 2
      .db TIPO_DISPARO, 38, 80, 3
      .db TIPO_ALEATORIO, 38, 120, 10
      .db 255,0,0,0 
      //66
      .db TIPO_POSTES, 62, 40, 80
      .db TIPO_POSTES, 62,120, 1
      .db TIPO_POSTES, 18, 32, 80
      .db TIPO_POSTES, 18,112, 1
      .db TIPO_EJES, 38, 160, 3
      .db 255,0,0,0 
      //67
      .db TIPO_CONCURSANTE_1, 38, 100, 1
      .db TIPO_DISPARO, 38, 100, 2
      .db TIPO_DISPARO, 38, 100, 1
      .db 255,0,0,0 
      //68
      .db TIPO_OJOS, 28, 160, 3
      .db TIPO_OJOS, 44, 160, 3
      .db TIPO_EJES, 38,100, 3
      .db TIPO_PERSEGUIDOR, 38, 100, 3
      .db 255,0,0,0 
      //69
      .db TIPO_CANNON, 72, 24, 3
      .db TIPO_CANNON, 72, 24, 2
      .db TIPO_PERSEGUIDOR, 38, 100, 3
      .db 255,0,0,0 
      //70
      .db TIPO_PINCHOS, 38, 100, 0
      .db TIPO_DISPARO, 38, 100, 3
      .db TIPO_PERS_ESTATICO, 60, 132, 4
      .db 255,0,0,0 
      //71
      .db TIPO_CANNON, 72, 24, 2
      .db TIPO_CANNON, 4, 24, 2
      .db TIPO_CANNON, 72, 24, 3
      .db TIPO_CANNON, 72, 144, 3
      .db 255,0,0,0 
   __endasm;
}

TEnemigo* enemigos_ptr;
u8 contador_int;
u8 bucle;

u8 contador_musica;
u8 velocidad_musica;

u8 tiempo;

u8 energia;
u8 energia_fin;

u8 game_over;

u16 score;
u16 hiscore;
u8 reloj_bonus;
u8 moneda_bonus;

u8 frames;
u8 stop;
u16 tileItems[8];
u16 bonus_medical;
u8 area;

void playMusic(){
   //El reproductor Arkos hace uso de los registros auxiliares (AF',BC',...) por lo que si luego se hace uso
   //de funciones que hagan uso de ellos (como easytiles) puede corromperse el programa si llega la interrupción a medio proceso.
   //Por eso, es mejor curarse en salud y guardarlos siempre de manera manual antes de llamar a la reproducción de la música
   __asm
      EXX
      .db 0x08 ;; //EX AF,AF' para que no salte warning al compilar por creerse que es una cadena de carácter sin cerrar
      PUSH AF
      PUSH BC
      PUSH DE
      PUSH HL
      call _PLY_AKG_PLAY
      POP HL
      POP DE
      POP BC
      POP AF
      .db 0x08 ;; //EX AF,AF'
      EXX
   __endasm;
}

void setInterruptAtVSYNCStart(void *intHdl){
   //Asegurarnos que cambiamos la gestión de interrupciones al terminar VSync
   //pues el primero puede saltar si estamos a mitad de VSync. Por eso se pasan 2 interrupciones más y entra en espera.
   //(halt paraliza la ejecución hasta que se produzca una nueva interrupción)
   //Ahora es seguro que saltará de la espera al llegar el CRTC al VSync
   cpct_waitVSYNC();
   __asm
      halt 
      halt
   __endasm;
   cpct_waitVSYNC(); 
   contador_int = 5;
   cpct_setInterruptHandler(intHdl);
}


void intHandler(){
   //cpct_setBorder (cpct_getHWColour (contador_int));
   
   if (!contador_int) {   
      contador_int = 6;
      bucle++;
   }

   if (musica && !contador_musica) {   
      contador_musica = velocidad_musica;
      playMusic();
   }

   contador_int--;
   contador_musica--;
}

void intro();

void pausa(){
  u8 i;

  //i = nº segundos
  for (i=0; i<3; i++) {
    cpct_waitHalts(255);
  }
}

void parpadeo() {

  if (ciclo == 0) 
    cpct_memcpy (&paleta[0],&paleta_original,16);
  if (ciclo == 3 && cpct_rand()%2)
    cpct_memcpy (&paleta[0],&paleta_oscura,16);
  if (ciclo == 6 && cpct_rand()%2)
    cpct_memcpy (&paleta[0],&paleta_original,16);
  if (ciclo == 8)
    cpct_memcpy (&paleta[0],&paleta_oscura,16);
  cpct_setPalette(paleta,16); 
}

void main(void) {
  cpct_setStackLocation ((u8*) 0x7FFF); //Cambiar el contador de pila (SP) a 0x8000

  intro();

  while (1) {
    //Inicializar doble buffer
    swap_memvideo = 0;
    mem_start = (u8*) 0xC000;
    mem_page = cpct_pageC0;

    menu();
    initGame();
    entering();
    game();
  }
}

CPCT_ABSOLUTE_LOCATION_AREA(0x9A40);


//year 2030, rich and poor are more
//distant than ever. a cruel tv show 
//is the last hope for many.          
//'deep, deeper, deepest'             
//where the prize is your life ...     
const char txt1[35] = {34,14,10,27,38, 2, 0, 3, 0,37,38,27,18,12,17,38,10,23,13,38,25,24,24,27,38,10,27,14,38,22,24,27,14,38,255};
const char txt2[35] = {13,18,28,29,10,23,29,38,29,17,10,23,38,14,31,14,27,36,38,10,38,12,27,30,14,21,38,29,31,38,28,17,24,32,255};
const char txt3[35] = {18,28,38,29,17,14,38,21,10,28,29,38,17,24,25,14,38,15,24,27,38,22,10,23,34,36,38,38,38,38,38,38,38,38,255};
const char txt4[35] = {40,13,14,14,25,37,38,13,14,14,25,14,27,37,38,13,14,14,25,14,28,29,40,38,38,38,38,38,38,38,38,38,38,38,255};
const char txt5[35] = {32,17,14,27,14,38,29,17,14,38,25,27,18,35,14,38,18,28,38,34,24,30,27,38,21,18,15,14,38,36,36,36,38,38,255};
//dare to enter this deadly maze and
//fight against other contestants
//to get all the money you can.
//collect the color keys and go 
//deeper to exit alive
const char txt6[35] = {13,10,27,14,38,29,24,38,14,23,29,14,27,38,29,17,18,28,38,13,14,10,13,21,34,38,22,10,35,14,38,10,23,13,255};
const char txt7[35] = {15,18,16,17,29,38,10,16,10,18,23,28,29,38,24,29,17,14,27,38,12,24,23,29,14,28,29,10,23,29,28,38,38,38,255};
const char txt8[35] = {29,24,38,16,14,29,38,10,21,21,38,29,17,14,38,22,24,23,14,34,38,34,24,30,38,12,10,23,36,38,38,38,38,38,255};
const char txt9[35] = {12,24,21,21,14,12,29,38,29,17,14,38,12,24,21,24,27,38,20,14,34,28,38,10,23,13,38,16,24,38,38,38,38,38,255};
const char txt0[35] = {13,14,14,25,14,27,38,29,24,38,14,33,18,29,38,10,21,18,31,14,36,38,38,38,38,38,38,38,38,38,38,38,38,38,255};

//energy
const char txtBoard1[7] = {14,23,14,27,16,34,255};
//timer
const char txtBoard2[6] = {29,18,22,14,27,255};
//map
const char txtBoard3[4] = {22,10,25,255};
//access
const char txtBoard4[7] = {10,12,12,14,28,28,255};
//keys
const char txtBoard5[6] = {20,14,34,28,38,255};
//press any key to continue
const char txtBoard6[26] = {25,27,14,28,28,38,10,23,34,38,20,14,34,38,29,24,38,12,24,23,29,18,23,30,14,255};
//.be the first to gain the prize
//in this crazy contest
//const char txtIntro_1[32] = {36,11,14,38,29,17,14,38,15,18,27,28,29,38,29,24,38,16,10,18,23,38,29,17,14,38,25,27,18,35,14,255};
//const char txtIntro_1_2[22] = {18,23,38,29,17,18,28,38,12,27,10,35,34,38,12,24,23,29,14,28,29,255};
//.you'll need these
const char txtIntro_2_1 [19] = {36,34,24,30,40,21,21,38,23,14,14,13,38,29,17,14,28,14,255};
//.survive in the rooms until timer gets
const char txtIntro_3 [39] = {36,28,30,27,31,18,31,14,38,18,23,38,29,17,14,38,27,24,24,22,28,38,30,23,29,18,21,38,29,18,22,14,27,38,16,14,29,28,255};
//.pick up
const char txtIntro_4 [9] = {36,25,18,12,20,38,30,25,255};
//to speed up timer
const char txtIntro_5 [18] = {29,24,38,28,25,14,14,13,38,30,25,38,29,18,22,14,27,255};
//bonus
const char txtIntro_6 [6] = {11,24,23,30,28,255};
//to get more points
const char txtIntro_7 [19] = {29,24,38,16,14,29,38,22,24,27,14,38,25,24,18,23,29,28,255};
//to health up,every 2000 pts
const char txtIntro_8 [28] = {29,24,38,17,14,10,21,29,17,38,30,25,37,14,31,14,27,34,38,2,0,0,0,38,25,29,28,255};

//2022 cperezgrin, greenweb & ricardo oyon
const char txtCredits_1 [41] = {2,0,2,2,38,12,25,14,27,14,35,16,27,18,23,37,38,16,27,14,14,23,32,14,11,38,42,38,27,18,12,10,27,13,24,38,24,34,24,23,255};
//2022 greenweb, ricardo oyon & cperezgrin 
const char txtCredits_2 [41] = {2,0,2,2,38,16,27,14,14,23,32,14,11,37,38,27,18,12,10,27,13,24,38,24,34,24,23,38,42,38,12,25,14,27,14,35,16,27,18,23,255};
//2022 ricardo oyon, cperezgrin & greenweb
const char txtCredits_3 [41] = {2,0,2,2,38,27,18,12,10,27,13,24,38,24,34,24,23,37,38,12,25,14,27,14,35,16,27,18,23,38,42,38,16,27,14,14,23,32,14,11,255};


void intro() {
  u8 i;
  u32 semilla;
  u8 text[6];
  u8 *ptr;

  cpct_disableFirmware(); //Desactivar firmware
  setInterruptAtVSYNCStart(intHandler);
  musica = 0;
  contador_musica = velocidad_musica = 6;
  //Arrancar música del menú
    __asm
      ld hl,#_MENU_START    ;The music.
      ld a, #0     ;The Subsong to play (>=0).
      call _PLY_AKG_INIT
    __endasm;

    musica = 1; //Decirle al INT que ya puede reproducir la música

  cpct_setVideoMode(0); //Modo de Vídeo 0: 160x200 pixeles; 16 colores en pantalla
  cpct_memcpy (&paleta[0],&paleta_original,16);
  cpct_setPalette(paleta,16); //Establecer paleta de colores (16 colores)
  hiscore = 0;

  cpct_setBorder(HW_BLACK);
  cpct_memset (CPCT_VMEM_START, cpct_px2byteM0 (0, 0), 0x4000);

  //Imágenes de Intro
  ptr = (u8*) intro1_end;
    cpct_zx7b_decrunch_s((void *) (0xFFFF),(void *) ptr);
    cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000,  8, 0),cpct_px2byteM0 (2, 2) ,60,44);
    cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000, 68, 0),cpct_px2byteM0 (2, 2) ,12,44);
    cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000,  9, 1),cpct_px2byteM0 (0, 0) ,60,42);
    cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000, 68, 1),cpct_px2byteM0 (0, 0) ,11,42);

    f_c = cpct_getScreenPtr (CPCT_VMEM_START, 10,2);
    f_teletipo = 2;
    myDrawString(txt1);
    f_teletipo = 2;
    myDrawString(txt2);
    f_teletipo = 2;
    myDrawString(txt3);
    f_teletipo = 2;
    myDrawString(txt4);
    f_teletipo = 2;
    myDrawString(txt5);
    pausa();



ptr = (u8*) intro2_end;
    cpct_zx7b_decrunch_s((void *) (0xFFFF),(void *) ptr);
    cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000,  8, 0),cpct_px2byteM0 (2, 2) ,60,44);
    cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000, 68, 0),cpct_px2byteM0 (2, 2) ,12,44);
    cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000,  9, 1),cpct_px2byteM0 (0, 0) ,60,42);
    cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000, 68, 1),cpct_px2byteM0 (0, 0) ,11,42);
    f_c = cpct_getScreenPtr (CPCT_VMEM_START, 10,2);
    f_teletipo = 2;
    myDrawString(txt6);
    f_teletipo = 2;
    myDrawString(txt7);
    f_teletipo = 2;
    myDrawString(txt8);
    f_teletipo = 2;
    myDrawString(txt9);
    f_teletipo = 2;
    myDrawString(txt0);
    pausa();

cpct_memset ((u8*) 0xC000, cpct_px2byteM0 (0, 0), 0x4000);
  //MARCADOR EJEMPLO
  cpct_drawSpriteMasked (G_Player_R_F1, cpctm_screenPtr ((u8*) 0xC000, 0, 0), 4, 16);
  cpct_drawSolidBox(cpctm_screenPtr ((u8*) 0xC000, 4, 4),cpct_px2byteM0 (13, 13) ,MAX_ENERGIA/2,8);
  cpct_drawSprite (&G_Timer_0[0], cpctm_screenPtr ((u8*) 0xC000, 35, 1), 4, 14);
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 40,0);
  f_teletipo = 2;
  myDrawString(txtScore);
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 40,8);
  f_teletipo = 2;
  myDrawString(txtHiScore);

  text[0] = 0;
  text[1] = 0;
  text[2] = 0;
  text[3] = 0;
  text[4] = 0;
  text[5] = 255;  
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 52,0);
  f_teletipo = 2;
  myDrawString(text);
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 52,8);
  f_teletipo = 2;
  myDrawString(text);

  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 62,  3), cpct_px2byteM0 (2, 2), 8, 9);
  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 64,  6), cpct_px2byteM0 (9, 9), 1, 1);

  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 71,  1), cpct_px2byteM0 (9, 0), 1, 4);
  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 71,  6), cpct_px2byteM0 (13, 0), 1, 4);
  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 71, 11), cpct_px2byteM0 (7, 0), 1, 4);

  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 72,0);
  f_teletipo = 2;
  myDrawString(txtRoom);

  text[0] = 43;
  text[1] = 43;
  text[2] = 255;  
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 75,8);
  f_teletipo = 2;
  myDrawString(text);


  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 4,  18), cpct_px2byteM0 (0, 2), 1, 8);
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 4,28);
  f_teletipo = 2;
  myDrawString(txtBoard1);

  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 36,  18), cpct_px2byteM0 (0, 2), 1, 8);
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 32,28);
  f_teletipo = 2;
  myDrawString(txtBoard2);

  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 65,  18), cpct_px2byteM0 (0, 2), 1, 8);
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 63,28);
  f_teletipo = 2;
  myDrawString(txtBoard3);

  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 70,  18), cpct_px2byteM0 (0, 2), 1,16);
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 68,36);
  f_teletipo = 2;
  myDrawString(txtBoard4);
  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 68,44);
  f_teletipo = 2;
  myDrawString(txtBoard5);

  /*f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 0,56);
  myDrawString(txtIntro_1); 
  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 0,64);
  myDrawString(txtIntro_1_2); */

  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 0,80);
  myDrawString(txtIntro_2_1); 
  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 36, 78), cpct_px2byteM0 (2, 2), 14, 12);
  cpct_drawSpriteMasked (&G_Card_R[0], cpctm_screenPtr (CPCT_VMEM_START, 37, 80), 3, 8);
  cpct_drawSpriteMasked (&G_Card_G[0], cpctm_screenPtr (CPCT_VMEM_START, 41, 80), 3, 8);
  cpct_drawSpriteMasked (&G_Card_B[0], cpctm_screenPtr (CPCT_VMEM_START, 45, 80), 3, 8);

  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 0,96);
  myDrawString(txtIntro_3); 
  cpct_drawSprite (&G_Timer_8[0], cpctm_screenPtr (CPCT_VMEM_START, 76, 92), 4, 14);


  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 0,112);
  myDrawString(txtIntro_4); 
  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 16, 110), cpct_px2byteM0 (2, 2), 14, 12);
  cpct_drawSpriteMasked (&G_Clock_1[0], cpctm_screenPtr (CPCT_VMEM_START, 17, 112), 4, 8);
  cpct_drawSpriteMasked (&G_Clock_2[0], cpctm_screenPtr (CPCT_VMEM_START, 21, 112), 4, 8);
  cpct_drawSpriteMasked (&G_Clock_3[0], cpctm_screenPtr (CPCT_VMEM_START, 25, 112), 4, 8);
  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 31,112);
  myDrawString(txtIntro_5); 
  
  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 0,130);
  myDrawString(txtIntro_4); 
  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 16, 128), cpct_px2byteM0 (2, 2), 14, 12);
  cpct_drawSpriteMasked (&G_Coin_1[0], cpctm_screenPtr (CPCT_VMEM_START, 17, 130), 4, 8);
  cpct_drawSpriteMasked (&G_Coin_2[0], cpctm_screenPtr (CPCT_VMEM_START, 21, 130), 4, 8);
  cpct_drawSpriteMasked (&G_Coin_3[0], cpctm_screenPtr (CPCT_VMEM_START, 25, 130), 4, 8);
  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 31,130);
  myDrawString(txtIntro_7); 

  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 0,146);
  myDrawString(txtIntro_4); 
  cpct_drawSolidBox (cpct_getScreenPtr ((u8*) 0xC000, 16, 144), cpct_px2byteM0 (2, 2), 5, 12);
  cpct_drawSpriteMasked (&G_Medical[0], cpctm_screenPtr (CPCT_VMEM_START, 17, 146), 3, 8);
  f_teletipo = 2;
  f_c = cpctm_screenPtr (CPCT_VMEM_START, 22,146);
  myDrawString(txtIntro_8); 

  f_c = cpct_getScreenPtr (CPCT_VMEM_START, 16,172);
  f_teletipo = 2;
  myDrawString(txtBoard6);
  
  do {
    cpct_scanKeyboard_f();
  } while (cpct_isAnyKeyPressed_f ());

  //Esperar a pulsación de tecla 
  semilla = 0;
  while (!cpct_isAnyKeyPressed_f()) {
    semilla++;
    cpct_scanKeyboard_f();
  }
  

  if (!semilla)
    semilla = 1;

  cpct_setSeed_mxor(semilla);
  i = cpct_getRandom_mxor_u8() % 3;

  f_c = cpctm_screenPtr (CPCT_VMEM_START, 0,192);
  f_teletipo = 2;
  if (i == 0)
    myDrawString(txtCredits_1); 

  if (i == 1)
    myDrawString(txtCredits_2); 

  if (i == 2)
    myDrawString(txtCredits_3); 


playFX(9);
cpct_waitHalts(255);
cpct_waitHalts(255);
  
  frames = 0;

  //Datos que no cambian durante la ejecución
  entidades[0].id = 1; 
  entidades[0].alto = 16;
  entidades[0].ancho = 4;
}